//
// Created by shushiri on 1.1.19.
//

#ifndef EXACT_COVERS_COMPUTATION_H
#define EXACT_COVERS_COMPUTATION_H



#include <string/LinearString.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/simplify/EpsilonRemoverIncoming.h>
#include <stringology/indexing/NondeterministicExactSuffixAutomaton.h>

namespace stringology{
namespace cover {

class ExactCoversComputation {
public:
    /**
     *
     * @return set of all exact covers of input pattern.
     */

    template < class SymbolType >
    static ext::set < string::LinearString < SymbolType > > compute(const string::LinearString < SymbolType > & pattern);
};



template < class SymbolType >
ext::set < string::LinearString < SymbolType > > ExactCoversComputation::compute( const string::LinearString< SymbolType > &pattern ) {

    automaton::EpsilonNFA< SymbolType, DefaultEpsilonType, unsigned > suffixNDAwithEpsilonTransitions =
            stringology::indexing::NondeterministicExactSuffixAutomaton::construct( pattern );

    automaton::NFA< SymbolType, unsigned > suffixNDA =
            automaton::simplify::EpsilonRemoverIncoming::remove( suffixNDAwithEpsilonTransitions );


    ext::set < unsigned > initialState;
    initialState.insert( suffixNDA.getInitialState() );
    ext::vector < SymbolType > inputString = pattern.getContent();
    ext::set < unsigned > previousState( std::move( initialState ) );

    ext::set < string::LinearString < SymbolType > > result;
    ext::vector < SymbolType > cover;


    for ( unsigned i = 0; i < inputString.size(); ++i ) {
        cover.push_back( inputString[i] );
        ext::set < unsigned > newState;

        for ( unsigned nfaState : previousState ) {
            auto iter = suffixNDA.getTransitions().find( ext::make_pair( std::move( nfaState ), inputString[i] ) );

            if ( iter != suffixNDA.getTransitions().end() )
                newState.insert( iter->second.begin(), iter->second.end() );
        }

        if ( newState.size() < 2 ) break;

        previousState = newState;
        ext::set<unsigned>::iterator it = newState.end();

        if ( ( *( --it ) ) == inputString.size() ) { // The state is final
            unsigned previous;
            int cnt = 0;
            bool isCover = false;
            for ( unsigned nfaState : newState ) {
                if ( cnt ) {
                    if ( ( nfaState - previous ) > cover.size() ) {
                        isCover = false;
                        break;
                    }
                    isCover = true;
                }
                previous = nfaState;
                cnt++;
            }
            if ( isCover )
                result.insert( string::LinearString<SymbolType> ( cover ) );
        }
    }
    return result;

}



} /* namespace cover */
}/* namespace stringology */


#endif /* EXACT_COVERS_COMPUTATION_H */
