//
// Created by shushiri on 18.2.19.
//

#include <alib/set>
#include <string/LinearString.h>
#include <stringology/cover/NondeterministicApproximateSuffixAutomatonForHammingDistance.h>
#include "NondeterministicApproximateSuffixAutomatonForHammingDistanceTest.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( NondeterministicApproximateSuffixAutomatonForHammingDistanceTest, "stringology");
CPPUNIT_TEST_SUITE_REGISTRATION( NondeterministicApproximateSuffixAutomatonForHammingDistanceTest );

void NondeterministicApproximateSuffixAutomatonForHammingDistanceTest::setUp() {
}

void NondeterministicApproximateSuffixAutomatonForHammingDistanceTest::tearDown() {
}

void NondeterministicApproximateSuffixAutomatonForHammingDistanceTest::testNondeterministicApproximateSuffixAutomatonForHammingDistance() {
    string::LinearString < char > pattern ( "abcc" );
    unsigned int k = 1;
    auto result = stringology::cover::NondeterministicApproximateSuffixAutomatonForHammingDistance::construct( pattern, k);

    auto state00 = ext::make_pair( 0, 0 );
    automaton::EpsilonNFA < char, DefaultEpsilonType, ext::pair < unsigned int, unsigned int > > ref ( state00 );
    ref.setInputAlphabet(ext::set < char > { 'a', 'b', 'c' });

    auto state10 = ext::make_pair( 1, 0 );
    auto state20 = ext::make_pair( 2, 0 );
    auto state30 = ext::make_pair( 3, 0 );
    auto state40 = ext::make_pair( 4, 0 );
    auto state11 = ext::make_pair( 1, 1 );
    auto state21 = ext::make_pair( 2, 1 );
    auto state31 = ext::make_pair( 3, 1 );
    auto state41 = ext::make_pair( 4, 1 );

    ext::set < ext::pair < unsigned int, unsigned int > > states = { state00, state10, state20, state30, state40, state11, state21, state31, state41 };
    ref.setStates( states );

    ref.setFinalStates( ext::set < ext::pair < unsigned int, unsigned int > > { state40, state41 });

    //first stage
    ref.addTransition( state00, 'a', state10 );
    ref.addTransition( state10, 'b', state20 );
    ref.addTransition( state20, 'c', state30 );
    ref.addTransition( state30, 'c', state40 );
    //second stage
    ref.addTransition( state11, 'b', state21 );
    ref.addTransition( state21, 'c', state31 );
    ref.addTransition( state31, 'c', state41 );
    //diagonal
    ref.addTransition( state00, 'b', state11 );
    ref.addTransition( state00, 'c', state11 );
    ref.addTransition( state10, 'a', state21 );
    ref.addTransition( state10, 'c', state21 );
    ref.addTransition( state20, 'a', state31 );
    ref.addTransition( state20, 'b', state31 );
    ref.addTransition( state30, 'a', state41 );
    ref.addTransition( state30, 'b', state41 );

    //epsilon
    CPPUNIT_ASSERT( ref.addTransition( state00, state10 ) == 1 );
    CPPUNIT_ASSERT( ref.addTransition( state00, state20 ) == 1 );
    ref.addTransition( state00, state30 );
    ref.addTransition( state00, state40 );

    CPPUNIT_ASSERT( ref.getStates().size() == 9 );
    CPPUNIT_ASSERT( result.getStates().size() == 9 );
    CPPUNIT_ASSERT( ref.getInputAlphabet().size() == 3 );
    CPPUNIT_ASSERT( result.getInputAlphabet().size() == 3 );
    CPPUNIT_ASSERT( ref.getEpsilonTransitions().size() < 2 );
    CPPUNIT_ASSERT( ref.getTransitions().size() == 16 );
    CPPUNIT_ASSERT( result.getTransitions().size() == 16 );
    CPPUNIT_ASSERT( ref == result );
}
