#include "ExceptionHandler.hpp"
#include <ostream>

namespace alib {

std::vector < std::function < int ( std::ostream &, unsigned ) > > & ExceptionHandler::handlers ( ) {
	static std::vector < std::function < int ( std::ostream &, unsigned ) > > res = {
		[ ] ( std::ostream & output, unsigned level ) {
			try {
				throw;
			} catch ( ... ) {
				output << level << ' ' << "unexpected exception." << std::endl;
				return 127;
			}
		}, [ ] ( std::ostream & output, unsigned level ) {
			try {
				throw;
			} catch ( const std::exception & e ) {
				output << level << ' ' << "standard exception: " << e.what() << std::endl;
				return rethrow_if_nested ( output, e, level, 3 );
			}
		}
	};
	return res;
}

int ExceptionHandler::rethrow_if_nested ( std::ostream & output, const std::exception & e, unsigned level, int result ) {
	try {
		std::rethrow_if_nested ( e );
		return result;
	} catch ( ... ) {
		return handlerRec ( output, level + 1, handlers ( ).rbegin ( ) );
	}
}

int ExceptionHandler::handlerRec ( std::ostream & output, unsigned level, const std::vector < std::function < int ( std::ostream &, unsigned ) > >::const_reverse_iterator & iter ) {
	if ( iter == handlers ( ).rend ( ) ) {
		output << level << "unhandled exception." << std::endl;
		return 127;
	}
	try {
		return ( * iter ) ( output, level );
	} catch ( ... ) {
		return handlerRec ( output, level, std::next ( iter ) );
	}
}

int ExceptionHandler::handle ( std::ostream & output ) {
	return handlerRec ( output, 0, handlers ( ).rbegin ( ) );
}

} /* namespace alib */
